"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const generic_filehandle_1 = require("generic-filehandle");
const nclist_1 = __importDefault(require("@gmod/nclist"));
const indexedfasta_1 = require("@gmod/indexedfasta");
// locals
const NCListFeature_1 = __importDefault(require("./NCListFeature"));
// create functions that fetch annotations from the server
// and return them in a format that can be used by the
// SeqPanel object
// this function is called by the SeqPanel component
// when it needs to fetch annotations and sequence for a given range
// it should return an object consisting of the following properties:
// feature: a feature object
// sequence: a sequence object (that has sequence for the range plus
// 	  some extra sequence on either side)
// 	  (see @gmod/indexedfasta for more info)
// 	  (see @gmod/nclist for more info)
// mode: a string that is one of:
//    genomic
//    genomic_sequence_updown
//    cds
//    cdna
//    protein
//    gene
//    gene_collapsed_intron
//    gene_updownstream
//    gene_updownstream_collapsed_intron
//    (though this will probably be overridden by the SeqPanel component)
// intronBp: the number of base pairs to show for truncated introns
function assembleBundle(props) {
    return __awaiter(this, void 0, void 0, function* () {
        const { fastaurl, refseq } = props;
        const feature = yield accessStore(props);
        const sequence = yield accessFasta(refseq, feature[1], feature[2], fastaurl);
        const f = new NCListFeature_1.default(feature);
        return {
            feature: f.toJSON(),
            sequence,
            mode: "protein",
            intronBp: 10,
        };
    });
}
exports.default = assembleBundle;
// Create accessFasta function
// this function is called by the assembleBundle function
// It is called with four arguments:
// refseq: the reference sequence (name, like "chr1")
// start: the start of the range (in interbase coordinates)
// end: the end of the range (in interbase coordinates)
// fastaURL: the URL of the fasta file
// it should return a sequence object consisting of:
// seq: the sequence for the range
// upstream: the sequence upstream of the range
// downstream: the sequence downstream of the range
function accessFasta(refseq, start, end, fastaURL) {
    return __awaiter(this, void 0, void 0, function* () {
        const fastaFilehandle = new generic_filehandle_1.RemoteFile(fastaURL);
        const faiFilehandle = new generic_filehandle_1.RemoteFile(fastaURL + ".fai");
        const gziFilehandle = new generic_filehandle_1.RemoteFile(fastaURL + ".gzi");
        const t = new indexedfasta_1.BgzipIndexedFasta({
            fasta: fastaFilehandle,
            fai: faiFilehandle,
            gzi: gziFilehandle,
            chunkSizeLimit: 500000,
        });
        const upstreamstart = start - 499;
        const downstreamend = end + 499;
        const seq = yield t.getSequence(refseq, start, end);
        const upstream = yield t.getSequence(refseq, upstreamstart, start);
        const downstream = yield t.getSequence(refseq, end, downstreamend);
        return {
            seq: seq || "",
            upstream: upstream || "",
            downstream: downstream || "",
        };
    });
}
// Create accessStore function
// this function is called by the assembleBundle function
// It is called with seven arguments:
// nclistbaseurl: the base URL of the NCList file
// urltemplate: the URL template for the NCList file
// refseq: the reference sequence (name, like "chr1")
// start: the start of the range (in interbase coordinates)
// end: the end of the range (in interbase coordinates)
// gene: the name of the gene
// transcript: the name of the transcript
// it should return a single transcript feature object
function accessStore(props) {
    var _a, e_1, _b, _c;
    return __awaiter(this, void 0, void 0, function* () {
        const { nclistbaseurl, urltemplate, refseq, start, end, gene, transcript } = props;
        const store = new nclist_1.default({
            urlTemplate: urltemplate,
            baseUrl: nclistbaseurl,
            readFile: (url) => new generic_filehandle_1.RemoteFile(url).readFile(),
        });
        try {
            for (var _d = true, _e = __asyncValues(store.getFeatures({
                refName: refseq,
                start: start,
                end: end,
            })), _f; _f = yield _e.next(), _a = _f.done, !_a;) {
                _c = _f.value;
                _d = false;
                try {
                    const feature = _c;
                    // keep only the transcript we're looking for
                    if (feature.get("name") === gene) {
                        return feature
                            .get("subfeatures")
                            .find((value) => value.get("name") === transcript);
                    }
                }
                finally {
                    _d = true;
                }
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (!_d && !_a && (_b = _e.return)) yield _b.call(_e);
            }
            finally { if (e_1) throw e_1.error; }
        }
    });
}
